"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ResourcePool = require("ts-resource-pool");
const factory_1 = require("./factory");
const debug = require('debug')('server-accepts-email');
class Client {
    constructor(server, senderDomain) {
        this.pool = new ResourcePool(new factory_1.default(server, senderDomain), 5);
    }
    test(email, { senderAddress }) {
        return this.pool.use(async (connection) => {
            await connection.execute(`MAIL FROM: <${senderAddress}>`).then((response) => {
                if (response.code !== 250)
                    throw new Error(`Server did not accept sender address: ${senderAddress}`);
            });
            const response = await connection.execute(`RCPT TO: <${email}>`);
            if (response.code === 250) {
                debug(`Server accepts email for "${email}"`);
                return { kind: 'answer', answer: true };
            }
            if (response.code === 550) {
                debug('The mailbox is unavailable');
                return { kind: 'answer', answer: false };
            }
            if (response.code === 553) {
                debug('The mailbox name is not allowed');
                return { kind: 'answer', answer: false };
            }
            if (response.code === 554 && response.comment.includes('this address does not exist')) {
                debug('The mailbox is unavailable (probably ProtonMail)');
                return { kind: 'answer', answer: false };
            }
            if (response.code === 501 && response.comment.includes('Bad recipient address syntax')) {
                debug('The mailbox name is not allowed (probably ProtonMail or ESMTP Postfix)');
                return { kind: 'answer', answer: false };
            }
            if (response.code === 504 && response.comment.includes('Recipient address rejected')) {
                debug('The mailbox name is not allowed (probably Yandex)');
                return { kind: 'answer', answer: false };
            }
            if (response.code === 501 && response.comment.includes(`<${email}>: `)) {
                debug('The mailbox name is not allowed (probably Runbox)');
                return { kind: 'answer', answer: false };
            }
            if (response.code === 450 && response.comment.includes('unknown user')) {
                debug('The mailbox is unavailable (probably ESMTP Postfix)');
                return { kind: 'answer', answer: false };
            }
            if (response.code === 554 && response.comment.includes('Invalid-Recipient')) {
                debug('The mailbox is unavailable (probably ESMTP Postfix)');
                return { kind: 'answer', answer: false };
            }
            if (response.code === 451 && response.comment.includes('https://community.mimecast.com/docs/DOC-1369#451')) {
                debug('Server is applying greylisting, estimated wait time: 60s');
                return { kind: 'greylist', timeout: 60 };
            }
            if (response.code === 450 && response.comment.includes('Greylisted, see http://postgrey.schweikert.ch/help/')) {
                debug('Server is applying greylisting, estimated wait time: 5m');
                return { kind: 'greylist', timeout: 300 };
            }
            if (response.code === 451 && response.comment.includes('is not yet authorized to deliver mail from')) {
                debug('Server is applying greylisting, estimated wait time: 10m');
                return { kind: 'greylist', timeout: 600 };
            }
            debug(`Unexpected response: ${response.code} - ${response.comment}`);
            throw new Error(`Unexpected code from server: ${response.code}`);
        });
    }
}
exports.default = Client;
